####################
# Zope Army Knife  #
# (c) 2003 NAUMEN  #
####################
"""
Revision information:
$Id: ZAK.py,v 1.6 2003/05/17 14:09:09 vpastukhov Exp $

ZAK class
"""
__version__ = '$Revision: 1.6 $'[11:-2]

from __future__ import nested_scopes

import sys, traceback

from Globals import DTMLFile, InitializeClass
from Acquisition import aq_inner, aq_parent
from AccessControl import ClassSecurityInfo 
from OFS.SimpleItem import SimpleItem

import ZAKContext
from ZAKContext import html_format


manage_addZAKForm = DTMLFile('dtml/addZAKForm', globals())

def manage_addZAK(self, id, add=None, REQUEST=None):
    "Create a new ZAK and add it to container"
    container = self.Destination()

    if add:
        zak = ZAK(id)
        container._setObject(id, zak)
 
    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.DestinationURL() + '/manage_main')


class ZAK(SimpleItem):
    """Zope Army Knife
    
    Zope arbitrary expression evaluator.
    """
    
    meta_type = 'ZAK (Zope Army Knife)'
    security = ClassSecurityInfo() 
    
    security.declareProtected('Use ZAK',
                              'index_html',
                              'manage_evaluate')

    _indexHtml = DTMLFile('dtml/indexHtml', globals())
    _evaluatorForm = DTMLFile('dtml/evaluatorForm', globals())

    manage_options = ({'label':'Evaluator', 'action':'manage_evaluate'},) + \
                     SimpleItem.manage_options

    def __init__(self, id):
        self.id = id

    def index_html( self, expr='', REQUEST=None ):
        """Evaluate any expression in any context"""
        if expr:
            result = self.evaluate( expr, REQUEST )
            result = html_format( result )
        else:
            result = ''

        return self._indexHtml( self, REQUEST, result=result, expr=expr )

    def manage_evaluate( self, expr='', REQUEST=None ):
        """Evaluate any expression in ZMI tab"""
        if expr:
            result = self.evaluate( expr, REQUEST )
            result = html_format( result )
        else:
            result = ''

        return self._evaluatorForm( self, REQUEST, result=result, expr=expr )

    def evaluate(self, expr, REQUEST=None):
        expr = expr.replace('\r\n', '\n') + '\n'
        locals = { 'REQUEST': REQUEST,
                   'context': aq_parent(self),
                   'app': self.getPhysicalRoot() }

        try:
            eval(compile(expr, 'code', 'exec'), ZAKContext.__dict__, locals)
        except:
            type, value, tb = sys.exc_info()
            return ''.join(traceback.format_exception_only(type, value) + \
                            ['\n'] + traceback.format_tb(tb)[1:])

        vars = locals.keys()
        vars.sort()

        try: vars.remove('REQUEST')
        except ValueError: pass

        return '\n'.join(map(lambda i: str(i) + ' = ' + str(locals[i]), vars))

    def parent(self):
        """return parent (context) of this ZAK"""
        return aq_parent(aq_inner(self))
        

InitializeClass(ZAK)
