####################
# Zope Army Knife  #
# (c) 2003 NAUMEN  #
####################
"""
Revision information:
$Id: ZAKContext.py,v 1.1 2003/05/17 14:09:09 vpastukhov Exp $

Namespace for code evaluation
"""
__version__ = "$Revision: 1.1 $"[11:-2]

# pre-import modules (for use in expressions)
import cgi, os, re, string, sys, types


def html_format( text ):
    text = cgi.escape( text ) \
              .replace( '  ', '&nbsp; ' ) \
              .replace( '  ', '&nbsp; ' ) \
              .replace( '\n', '<br />\n' )
    return text

def class_tree( klass, depth=None, indent=0 ):
    if indent:
        result = ''
    else:
        result = 'Inheritance tree of class %s:\n\n' % klass.__name__

    stack = [ [ klass ] ]

    while stack:
        classes = stack[-1]
        if not classes:
            stack.pop()
            continue

        level = len(stack) + indent
        klass = classes.pop(0)

        mark = klass.__bases__ and '+' or '-'
        mod  = getattr( klass, '__module__', '' )
        name = ( mod and mod + '.' ) + getattr( klass, '__name__', repr(klass) )

        result += '%s%s %s\n' % ( '  '*level, mark, name )

        if klass.__bases__ and (depth is None or level < depth):
            stack.append( list( klass.__bases__ ) )

    return result

def attr_tree( klass, attr, indent=0 ):
    if indent:
        result = ''
    else:
        result = 'Inheritance tree of attribute %s.%s:\n\n' % ( klass.__name__, attr )

    stack = [ [ klass ] ]
    seen  = 0

    while stack:
        classes = stack[-1]
        if not classes:
            stack.pop()
            continue

        level = len(stack) + indent
        klass = classes.pop(0)

        if not hasattr( klass, attr ):
            continue

        if klass.__dict__.has_key( attr ):
            mark = seen and '#' or '@'
            seen += 1
        else:
            mark = '+'

        mod  = getattr( klass, '__module__', '' )
        name = ( mod and mod + '.' ) + getattr( klass, '__name__', repr(klass) )

        result += '%s%s %s\n' % ( '  '*level, mark, name )

        if klass.__bases__:
            stack.append( list( klass.__bases__ ) )

    return result

def _getattrs( cls, attrs=None, lev=0 ):
    if attrs is None:
        attrs = {}

    for attr, val in cls.__dict__.items():
        if not attrs.has_key( attr ):
            attrs[ attr ] = []
        attrs[ attr ].append( (lev, cls, val) )

    for base in cls.__bases__:
        _getattrs( base, attrs, lev+1 )

    return attrs

_skipattrs = (
        '__doc__', '__ac_permissions__',
        '__basicnew__', '__reduce__', '__roles__',
        'inheritedAttribute',
    )

_skipregex = r'.*__roles__$'

_skipre = re.compile( _skipregex )

def attr_conflicts( klass, indent=0 ):
    if indent:
        result = ''
    else:
        result = 'Contradictions of inherited attributes in class %s:\n\n' % klass.__name__
    indent += 1

    attrs  = _getattrs( klass )
    sorted = attrs.keys()
    sorted.sort( lambda a, b: cmp( a.lower(), b.lower() ) )

    for attr in sorted:
        if attr in _skipattrs or _skipre.match( attr ):
            continue

        lst = attrs[ attr ]
        lev0, kls0, val0 = lst[0]
        count = 1
        seen  = {}

        for levi, klsi, vali in lst:
            if not ( seen.get( klsi ) or klsi is kls0 or \
                     levi > lev0 or vali is val0 ):
                count += 1
            seen[ klsi ] = 1

        if count > 1:
            result += '%s%s (%s)\n' % ( '  '*indent, attr, type(val0).__name__ )
            result += attr_tree( klass, attr, indent=indent )
            result += '\n'

    return result
